// Author 2012-2017 Serge Goncharov
/* This program is free software and is distributed under the terms of CC0 public domain license.
   See http://creativecommons.org/publicdomain/zero/1.0/ for details. */

#include <stdio.h>
#include <ctype.h>

#define BUFSIZE	(1024*1024)
#define MAX_FILES 256



static unsigned char buffer[BUFSIZE];
static const char usage[] =
    "Usage: bin2c [options] <filename> ...\n"
    "  Dumps the contents of files as C-source hex arrays.\n"
    "Options:\n"
    "  -o NAME  Set output file name\n";



int main (int argc, char **argv) {
  FILE *f, *dump;
  unsigned char c;
  unsigned long j, sz;
  char *ofile=NULL, *files[MAX_FILES];
  int i, filecnt=0;

  // Process options
  if (argc<2) {
    printf (usage);
    return 1;
  }

  for (i=1; i<argc; ++i) {
    if (argv[i][0]!='-') {
      if (filecnt<MAX_FILES)  files[filecnt++] = argv[i];
    } else switch (argv[i][1]) {
    case 'o':
      if (ofile!=NULL) {
	printf ("Error: More than one output file specified.\n");
	return 1;
      }
      j = 0;
      if (argv[i][2]!=0) {
	j = 2;
      } else if (++i>=argc) {
	printf (usage);
	return 1;
      }
      ofile = &argv[i][j];
      break;

    default:
      printf (usage);
      return 1;
    }
  }

  if (ofile==NULL) {
    printf ("Error: Output file not specified.\n");
    printf (usage);
    return 1;
  }

  dump = fopen (ofile, "wb");
  if (dump==NULL) {
    printf ("Can't create dump file.\n");
    return 1;
  }

  fprintf (dump, "/* Do not edit this file! It is automatically generated from the following file%s:\n   ", filecnt>1 ? "s" : "");
  for (i=0; i<filecnt-1; ++i)  fprintf (dump, "%s, ", files[i]);
  fprintf (dump, "%s */", files[i]);

  for (i=0; i<filecnt; ++i) {
    f = fopen (files[i], "rb");
    if (f==NULL) {
      printf ("Can't open file '%s'.\n", files[i]);
      continue;
    }

    fprintf (dump, "\n\n");
    for (j=0; (c=files[i][j])!=0; ++j) {
      c = (unsigned char)tolower(c);
      if (!isalnum(c))  c = '_';
      files[i][j] = c;
    }
    fprintf (dump, "unsigned char %s[] = {", files[i]);
    do {
      sz = fread (buffer, 1, BUFSIZE, f);
      for (j=0; j<sz; ++j) {
	if (j%16==0)  fprintf (dump, "\n ");
	fprintf (dump, " 0x%02X,", buffer[j]);
      }
    } while (sz==BUFSIZE);
    fprintf (dump, "\n};");
    fclose (f);
  }

  fclose (dump);
  return 0;
}
